﻿Imports System.ComponentModel
Imports System.Text
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel

' This sample demostrates the use of a stored procedure query asynchronously

Namespace SilverlightConsole
  Public Class CodeSample
	Public Sub New()
	End Sub

    Public Sub Start()
      Coroutine.Start(Function() AsyncStoredProcQueryCore(_entityManager))
    End Sub

    Private Function AsyncStoredProcQueryCore(ByVal manager As NorthwindIBEntities) As IEnumerable(Of Func(Of INotifyCompleted))
      ' Execute a stored procedure query asynchronously
      ' Because Silverlight is asynchronous, we use the coroutine syntax to avoid nesting callbacks within callbacks
      ' Note that coroutines are not a part of nor are necessary for executing a stored procedure query.
      ' If you're not familiar with coroutines, please disregard the "yield return's" and focus on the logic of the methods below.

      ' Setup of the list of async functions for the Coroutine to execute serially.
      Dim operationList = New List(Of Func(Of INotifyCompleted))

      Dim queryop
      Dim storedProcQueryOp


      Dim f1 As Func(Of INotifyCompleted) =
        Function()
          SLConsole.WriteLine("Note: The server may take a few seconds to initialize." & vbLf)

          '' Get an Employee so we can pass his ID as a parameter in the stored procedure query
          queryop = manager.Employees.OrderBy(Function(e) e.LastName).AsScalarAsync().First()
          Return queryop
        End Function
      operationList.Add(f1)

      Dim f2 As Func(Of INotifyCompleted) =
          Function()
            Dim employee As Employee = queryop.Result
            SLConsole.WriteLine("The employee retrieved is " & employee.FirstName & " " & employee.LastName & "; ID# " & employee.EmployeeID)
            SLConsole.WriteLine("Retrieving " & employee.FirstName & " " & employee.LastName & "'s orders via Stored Procedure Query.")

            ' We can pass the stored proc arguments as parameters to the query.  
            ' Here we'll query for orders for the employee retrieved in the previous query in the year 1996.
            Dim storedProcQuery As StoredProcQuery = manager.EmployeeOrdersForYearQuery(employee.EmployeeID, 1996)
            storedProcQueryOp = storedProcQuery.ExecuteAsync()
            Return storedProcQueryOp
          End Function
      operationList.Add(f2)

      Dim f3 As Func(Of INotifyCompleted) =
          Function()
            SLConsole.WriteLine("Retrieved " & storedProcQueryOp.Results.Count() & " orders:")

            ' The StoredProcQuery is an IEntityQuery, not an IEntityQuery<T>, so we need to cast the untyped results.
            For Each result In storedProcQueryOp.Results
              Dim o As Order = CType(result, Order)
              SLConsole.WriteLine("Order " & o.OrderID)
            Next result

            ' Let's also show navigation properties here.  We can't do an Include with the StoredProcQuery, but we can lazily and 
            ' asynchronously load related data. In Silverlight, asynchronous navigation is used to load related data into cache.  
            ' Note that we'll issue two async queries here (via async navigation). Since the queries will complete independent 
            ' of one another, the message list showing the results will not be in any particular order.

            Dim orders = New List(Of Order)
            For Each o In storedProcQueryOp.Results
              orders.Add(CType(o, Order))
            Next

            Dim firstOrder = orders(0)
            SLConsole.WriteLine(vbLf & "Retrieving Customer and Line items for first order (" & firstOrder.OrderID & ")...")

            ' Let's get the Customer for the first Order

            Dim cust As Customer = firstOrder.Customer
            AddHandler cust.EntityAspect.PendingEntityResolved, Sub(o, e) SLConsole.WriteLine("Retrieved customer '" & (CType(e.ResolvedEntity, Customer)).CompanyName & "'")


            ' We'll also retrieve the OrderDetails for the first order. We will listen on the PendingEntityListResolved event for such.
            Dim details As RelatedEntityList(Of OrderDetail) = firstOrder.OrderDetails
            AddHandler details.PendingEntityListResolved, Sub(o, e)
                                                            SLConsole.WriteLine("Retrieved " & e.ResolvedEntities.Count & " line items:")
                                                            For Each orderDetail In firstOrder.OrderDetails
                                                              SLConsole.WriteLine("Line item " & orderDetail.OrderID & "," & orderDetail.ProductID)
                                                            Next orderDetail
                                                          End Sub


            Return Coroutine.Return(Nothing)
          End Function
      operationList.Add(f3)

      Return operationList
    End Function

    Private _entityManager As New NorthwindIBEntities()
  End Class
End Namespace
